const webpack = require('webpack');
const chalk = require('chalk');
const appConfig = require('../common/appConfig');
const { runCallback } = require('../common/build');
const { getProjectPath, deleteFolders } = require('../common/utils');
const getEnvironmentInfo = require('../common/extendEnv');
const clientConfigFactory = require('./webpack.prod.client');
const serverConfigFactory = require('./webpack.prod.server');

process.env.BABEL_ENV = 'production';
process.env.NODE_ENV = 'production';

process.on('unhandledRejection', err => {
  // eslint-disable-next-line @typescript-eslint/no-throw-literal
  throw err;
});

const extendEnv = getEnvironmentInfo();

function build() {
  const buildClientOutput = getProjectPath(appConfig.buildClientOutput.path);
  deleteFolders(buildClientOutput);

  const buildServerOutput = getProjectPath(appConfig.buildServerOutput.path);
  deleteFolders(buildServerOutput);

  const clientConfig = clientConfigFactory(appConfig, extendEnv);
  const serverConfig = serverConfigFactory(appConfig, extendEnv);
  // eslint-disable-next-line no-console
  console.log(chalk.green('Creating an optimized production build...'));
  const clientCompiler = webpack(clientConfig);
  const serverCompiler = webpack(serverConfig);
  const clientCompilerPromise = new Promise((resolve, reject) => {
    clientCompiler.run(runCallback(resolve, reject));
  });
  const serverCompilerPromise = new Promise((resolve, reject) => {
    serverCompiler.run(runCallback(resolve, reject));
  });

  return Promise.all([clientCompilerPromise, serverCompilerPromise]);
}

build()
  .then(
    ([clientInfo, serverInfo]) => {
      if (clientInfo.warnings.length || serverInfo.warnings.length) {
        // eslint-disable-next-line no-console
        console.log(chalk.yellow('Compiled with warnings.\n'));
        // eslint-disable-next-line no-console
        console.log(clientInfo.warnings.join('\n\n'));
        // eslint-disable-next-line no-console
        console.log(serverInfo.warnings.join('\n\n'));
        // eslint-disable-next-line no-console
        console.log(`\nSearch for the ${chalk.underline(chalk.yellow('keywords'))} to learn more about each warning.`);
        // eslint-disable-next-line no-console
        console.log(`To ignore, add ${chalk.cyan('// eslint-disable-next-line')} to the line before.\n`);
      } else {
        // eslint-disable-next-line no-console
        console.log(chalk.green('Compiled successfully.\n'));
      }
    },
    err => {
      // eslint-disable-next-line no-console
      console.log(chalk.red('Failed to compile.\n'));
      // eslint-disable-next-line no-console
      console.log(err);
      process.exit(1);
    }
  )
  .catch(err => {
    // eslint-disable-next-line no-console
    console.log(chalk.red('Failed to compile.\n'));
    // eslint-disable-next-line no-console
    console.log(err);
    process.exit(1);
  });
